from typing import Dict, List, Optional, Tuple, Union, Any
from typing_extensions import Annotated
from pydantic import StrictFloat, StrictStr, StrictInt, StrictBytes, Field, validate_call

from smallestai.atoms.configuration import Configuration
from smallestai.atoms.api.agents_api import AgentsApi
from smallestai.atoms.api.knowledge_base_api import KnowledgeBaseApi
from smallestai.atoms.api.calls_api import CallsApi
from smallestai.atoms.models.create_agent_request import CreateAgentRequest
from smallestai.atoms.models.agent_id_patch_request import AgentIdPatchRequest
from smallestai.atoms.models.knowledgebase_post_request import KnowledgebasePostRequest
from smallestai.atoms.models.knowledgebase_id_items_upload_text_post_request import KnowledgebaseIdItemsUploadTextPostRequest
from smallestai.atoms.models.conversation_outbound_post_request import ConversationOutboundPostRequest
from smallestai.atoms.api_client import ApiClient
from smallestai.atoms.api.user_api import UserApi
from smallestai.atoms.api.organization_api import OrganizationApi
from smallestai.atoms.models.user_get200_response import UserGet200Response
from smallestai.atoms.models.organization_get200_response import OrganizationGet200Response
from smallestai.atoms.api.campaigns_api import CampaignsApi
from smallestai.atoms.models.campaign_post_request import CampaignPostRequest
from smallestai.atoms.models.campaign_get_request import CampaignGetRequest
from smallestai.atoms.api.agent_templates_api import AgentTemplatesApi
from smallestai.atoms.api.logs_api import LogsApi
from smallestai.atoms.models.create_agent_from_template_request import CreateAgentFromTemplateRequest
from smallestai.atoms.models.conversation_id_get200_response import ConversationIdGet200Response
# New modular managers (flat structure)
from smallestai.atoms.call import Call
from smallestai.atoms.audience import Audience
from smallestai.atoms.campaign import Campaign
from smallestai.atoms.kb import KB

class AtomsClient:
    def __init__(self, configuration=None) -> None:
        if configuration is None:
            configuration = Configuration.get_default()
        
        self.api_client = ApiClient(configuration)
        self.agents_api = AgentsApi(self.api_client)
        self.knowledge_base_api = KnowledgeBaseApi(self.api_client)
        self.calls_api = CallsApi(self.api_client)
        self.user_api = UserApi(self.api_client)
        self.organization_api = OrganizationApi(self.api_client)
        self.campaigns_api = CampaignsApi(self.api_client)
        self.agent_templates_api = AgentTemplatesApi(self.api_client)
        self.logs_api = LogsApi(self.api_client)
        
        # Initialize modular managers
        base_url = self.api_client.configuration.host
        self._call = Call(base_url)
        self._audience = Audience(base_url)
        self._campaign = Campaign(base_url)
        self._kb = KB(base_url)

    # ================================================================
    # Module Properties (Flat API)
    # ================================================================

    @property
    def call(self):
        """Call management and analytics."""
        return self._call
    
    @property
    def audience(self):
        """Audience management."""
        return self._audience

    @property
    def campaign(self):
        """Campaign management."""
        return self._campaign
        
    @property
    def kb(self):
        """Knowledge Base management."""
        return self._kb

    # Agent Methods
    @validate_call
    def create_agent(
        self,
        create_agent_request: CreateAgentRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.agents_api.agent_post(
            create_agent_request=create_agent_request,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    def delete_agent(
        self,
        id: str,
        **kwargs
    ):
        """
        Delete (archive) an agent by ID.
        
        Note: Backend uses /agent/{id}/archive endpoint.
        """
        import requests
        url = f"{self._get_base_url()}/agent/{id}/archive"
        headers = self._get_auth_headers()
        
        response = requests.delete(url, headers=headers)
        response.raise_for_status()
        return response.json()

    def get_agent_by_id(
        self,
        id: str,
        **kwargs
    ) -> Dict[str, Any]:
        """
        Get agent by ID with full details.
        
        Returns:
            Dict with agent details including globalKnowledgeBaseId
        """
        import requests
        url = f"{self._get_base_url()}/agent/{id}"
        headers = self._get_auth_headers()
        
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.json()

    @validate_call
    def get_agents(
        self,
        page: Optional[StrictInt] = None,
        offset: Optional[StrictInt] = None,
        search: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.agents_api.agent_get(
            page=page,
            offset=offset,
            search=search,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    @validate_call
    def update_agent(
        self,
        id: StrictStr,
        agent_id_patch_request: AgentIdPatchRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.agents_api.agent_id_patch(
            id=id,
            agent_id_patch_request=agent_id_patch_request,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    @validate_call
    def get_agent_workflow(
        self,
        id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.agents_api.agent_id_workflow_get(
            id=id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    # Knowledge Base Methods
    @validate_call
    def create_knowledge_base(
        self,
        knowledgebase_post_request: KnowledgebasePostRequest,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.knowledge_base_api.knowledgebase_post(
            knowledgebase_post_request=knowledgebase_post_request,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    @validate_call
    def delete_knowledge_base(
        self,
        id: StrictStr,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.knowledge_base_api.knowledgebase_id_delete(
            id=id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    @validate_call
    def get_knowledge_base_by_id(
        self,
        id: StrictStr,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.knowledge_base_api.knowledgebase_id_get(
            id=id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    @validate_call
    def get_knowledge_bases(
        self,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.knowledge_base_api.knowledgebase_get(
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    @validate_call
    def upload_text_to_knowledge_base(
        self,
        id: StrictStr,
        knowledgebase_id_items_upload_text_post_request: KnowledgebaseIdItemsUploadTextPostRequest,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.knowledge_base_api.knowledgebase_id_items_upload_text_post(
            id=id,
            knowledgebase_id_items_upload_text_post_request=knowledgebase_id_items_upload_text_post_request,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    @validate_call
    def upload_media_to_knowledge_base(
        self,
        id: StrictStr,
        media: Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]],
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.knowledge_base_api.knowledgebase_id_items_upload_media_post(
            id=id,
            media=media,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    @validate_call
    def get_knowledge_base_items(
        self,
        id: StrictStr,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.knowledge_base_api.knowledgebase_id_items_get(
            id=id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    @validate_call
    def delete_knowledge_base_item(
        self,
        knowledge_base_id: StrictStr,
        knowledge_base_item_id: StrictStr,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.knowledge_base_api.knowledgebase_knowledge_base_id_items_knowledge_base_item_id_delete(
            knowledge_base_id=knowledge_base_id,
            knowledge_base_item_id=knowledge_base_item_id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    # Calls Methods
    @validate_call
    def start_outbound_call(
        self,
        conversation_outbound_post_request: ConversationOutboundPostRequest,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.calls_api.conversation_outbound_post(
            conversation_outbound_post_request=conversation_outbound_post_request,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    # User Methods
    @validate_call
    def get_current_user(
        self,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UserGet200Response:
        return self.user_api.user_get(
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    # Organization Methods
    @validate_call
    def get_organization(
        self,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> OrganizationGet200Response:
        return self.organization_api.organization_get(
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    # Campaign Methods
    @validate_call
    def create_campaign(
        self,
        campaign_post_request: CampaignPostRequest,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.campaigns_api.campaign_post(
            campaign_post_request=campaign_post_request,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    @validate_call
    def delete_campaign(
        self,
        id: StrictStr,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.campaigns_api.campaign_id_delete(
            id=id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    def get_campaign_by_id(
        self,
        id: str,
        **kwargs
    ) -> Dict[str, Any]:
        """
        Get campaign by ID with full status and metrics.
        
        Returns:
            Dict with campaign details, status, and metrics
        """
        import requests
        url = f"{self._get_base_url()}/campaign/{id}"
        headers = self._get_auth_headers()
        
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.json()

    @validate_call
    def get_campaigns(
        self,
        campaign_get_request: CampaignGetRequest,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.campaigns_api.campaign_get(
            campaign_get_request=campaign_get_request,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    def start_campaign(
        self,
        id: str,
        **kwargs
    ) -> Dict[str, Any]:
        """
        Start a campaign by ID.
        
        Returns:
            Dict with message, taskId, and campaignId
        """
        import requests
        url = f"{self._get_base_url()}/campaign/{id}/start"
        headers = self._get_auth_headers()
        
        response = requests.post(url, headers=headers)
        response.raise_for_status()
        return response.json()

    @validate_call
    def pause_campaign(
        self,
        id: StrictStr,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.campaigns_api.campaign_id_pause_post(
            id=id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    # Agent Template Methods
    @validate_call
    def get_agent_templates(
        self,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.agent_templates_api.agent_template_get(
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    @validate_call
    def create_agent_from_template(
        self,
        create_agent_from_template_request: CreateAgentFromTemplateRequest,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ):
        return self.agent_templates_api.agent_from_template_post(
            create_agent_from_template_request=create_agent_from_template_request,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    # Logs Methods
    @validate_call
    def get_conversation_logs(
        self,
        id: StrictStr,
        _request_timeout: Union[None, Annotated[StrictFloat, Field(gt=0)], Tuple[Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]]] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ConversationIdGet200Response:
        return self.logs_api.conversation_id_get(
            id=id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

    # ================================================================
    # Call Analytics Methods (using direct HTTP calls for JSON response)
    # ================================================================
    
    def get_call(self, call_id: str) -> Dict[str, Any]:
        """
        Get details for a single call by call ID.
        
        Args:
            call_id: The call ID (e.g., "CALL-1768155029217-0bae45")
        
        Returns:
            Dict with status and data containing call details including:
            - callId, duration, status, type
            - transcript, recordingUrl, recordingDualUrl
            - from, to, events, callCost
        """
        import requests
        import os
        api_key = os.environ.get("SMALLEST_API_KEY", "")
        url = f"{self.api_client.configuration.host}/conversation/{call_id}"
        
        response = requests.get(url, headers={"Authorization": f"Bearer {api_key}"})
        response.raise_for_status()
        return response.json()
    
    def get_calls(
        self,
        agent_id: Optional[str] = None,
        campaign_id: Optional[str] = None,
        page: int = 1,
        limit: int = 10,
        status: Optional[str] = None,
        call_type: Optional[str] = None,
        search: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        Get paginated list of calls with optional filters.
        
        Args:
            agent_id: Filter by agent ID
            campaign_id: Filter by campaign ID
            page: Page number (default: 1)
            limit: Results per page (default: 10)
            status: Filter by status (completed, failed, in_progress, etc.)
            call_type: Filter by type (telephony_inbound, telephony_outbound, chat)
            search: Search by callId, fromNumber, or toNumber
        
        Returns:
            Dict with status and data containing paginated call logs
        """
        import requests
        import os
        api_key = os.environ.get("SMALLEST_API_KEY", "")
        url = f"{self.api_client.configuration.host}/conversation"
        
        params = {"page": page, "limit": limit}
        if agent_id:
            params["agentIds"] = agent_id
        if campaign_id:
            params["campaignIds"] = campaign_id
        if status:
            params["statusFilter"] = status
        if call_type:
            params["callTypes"] = call_type
        if search:
            params["search"] = search
        
        response = requests.get(url, headers={"Authorization": f"Bearer {api_key}"}, params=params)
        response.raise_for_status()
        return response.json()
    
    def search_calls(self, call_ids: List[str]) -> Dict[str, Any]:
        """
        Search for multiple calls by their call IDs.
        
        Args:
            call_ids: List of call IDs to fetch (max 100)
        
        Returns:
            Dict with status and data containing matching call logs
        """
        import requests
        import os
        api_key = os.environ.get("SMALLEST_API_KEY", "")
        url = f"{self.api_client.configuration.host}/conversation/search"
        
        response = requests.post(
            url,
            headers={"Authorization": f"Bearer {api_key}", "Content-Type": "application/json"},
            json={"callIds": call_ids}
        )
        response.raise_for_status()
        return response.json()

    # ================================================================
    # Post-Call Analytics Methods
    # ================================================================
    
    def get_post_call_config(self, agent_id: str) -> Dict[str, Any]:
        """
        Get post-call analytics configuration for an agent.
        
        Args:
            agent_id: The agent ID
        
        Returns:
            Dict with status and data containing:
            - summaryPrompt: Custom prompt for generating call summaries
            - dispositionMetrics: List of configured disposition metrics
        """
        import requests
        import os
        api_key = os.environ.get("SMALLEST_API_KEY", "")
        url = f"{self.api_client.configuration.host}/agent/{agent_id}/post-call-analytics"
        
        response = requests.get(url, headers={"Authorization": f"Bearer {api_key}"})
        response.raise_for_status()
        return response.json()
    
    def set_post_call_config(
        self,
        agent_id: str,
        summary_prompt: str = None,
        disposition_metrics: List[Dict[str, Any]] = None
    ) -> Dict[str, Any]:
        """
        Set post-call analytics configuration for an agent.
        
        Args:
            agent_id: The agent ID
            summary_prompt: Custom prompt for generating call summaries
            disposition_metrics: List of disposition metrics to track. Each metric should have:
                - identifier: Unique ID (e.g., "customer_status")
                - dispositionMetricPrompt: Question to extract this data
                - dispositionValues: Dict with "type" key (STRING, BOOLEAN, INTEGER, ENUM, DATETIME)
                - choices: List of options (required only for ENUM type)
        
        Returns:
            Dict with status and updated configuration
        
        Example:
            client.set_post_call_config(
                agent_id="123",
                summary_prompt="Summarize the key outcomes of this call.",
                disposition_metrics=[
                    {
                        "identifier": "customer_status",
                        "dispositionMetricPrompt": "What is the customer's interest level?",
                        "dispositionValues": {"type": "ENUM"},
                        "choices": ["Interested", "Not Interested", "Callback"]
                    },
                    {
                        "identifier": "customer_name",
                        "dispositionMetricPrompt": "What is the customer's name?",
                        "dispositionValues": {"type": "STRING"}
                    }
                ]
            )
        """
        import requests
        import os
        api_key = os.environ.get("SMALLEST_API_KEY", "")
        url = f"{self.api_client.configuration.host}/agent/{agent_id}/post-call-analytics"
        
        payload = {}
        if summary_prompt is not None:
            payload["summaryPrompt"] = summary_prompt
        if disposition_metrics is not None:
            payload["dispositionMetrics"] = disposition_metrics
        
        response = requests.post(
            url,
            headers={"Authorization": f"Bearer {api_key}", "Content-Type": "application/json"},
            json=payload
        )
        response.raise_for_status()
        return response.json()

    # ================================================================
    # Audience Methods (using direct HTTP calls)
    # ================================================================
    
    def _get_auth_headers(self) -> Dict[str, str]:
        """Get authorization headers for direct API calls."""
        import os
        api_key = os.environ.get("SMALLEST_API_KEY", "")
        return {
            "Authorization": f"Bearer {api_key}",
            "Content-Type": "application/json"
        }
    
    def _get_base_url(self) -> str:
        """Get base API URL."""
        return self.api_client.configuration.host
    
    def get_audiences(self) -> Dict[str, Any]:
        """
        Get all audiences for the organization.
        
        Returns:
            Dict with status and data containing list of audiences
        """
        import requests
        url = f"{self._get_base_url()}/audience"
        headers = self._get_auth_headers()
        
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.json()
    
    def get_audience_by_id(self, audience_id: str) -> Dict[str, Any]:
        """
        Get audience by ID.
        
        Args:
            audience_id: The audience ID
            
        Returns:
            Dict with status and audience data
        """
        import requests
        url = f"{self._get_base_url()}/audience/{audience_id}"
        headers = self._get_auth_headers()
        
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.json()
    
    def create_audience(
        self,
        name: str,
        phone_numbers: List[str],
        phone_number_column_name: str = "phoneNumber",
        description: str = "",
        names: Optional[List[Tuple[str, str]]] = None
    ) -> Dict[str, Any]:
        """
        Create a new audience with phone numbers.
        
        Args:
            name: Name of the audience
            phone_numbers: List of phone numbers to add (e.g., ["+916366821717", "+919353662554"])
            phone_number_column_name: Column name for phone numbers in CSV (default: "phoneNumber")
            description: Optional description
            names: Optional list of (firstName, lastName) tuples matching phone_numbers
            
        Returns:
            Dict with status and created audience data including _id
            
        Example:
            audience = client.create_audience(
                name="My Audience",
                phone_numbers=["+916366821717", "+919353662554"],
                names=[("John", "Doe"), ("Jane", "Smith")]
            )
            audience_id = audience["data"]["_id"]
        """
        import requests
        import io
        
        url = f"{self._get_base_url()}/audience"
        headers = {"Authorization": self._get_auth_headers()["Authorization"]}
        
        # Build CSV content
        csv_lines = ["firstName,lastName,phoneNumber"]
        for i, phone in enumerate(phone_numbers):
            if names and i < len(names):
                first_name, last_name = names[i]
            else:
                first_name, last_name = "User", str(i + 1)
            csv_lines.append(f"{first_name},{last_name},{phone}")
        
        csv_content = "\n".join(csv_lines)
        
        files = {'file': ('contacts.csv', csv_content.encode('utf-8'), 'text/csv')}
        data = {
            'name': name,
            'phoneNumberColumnName': phone_number_column_name,
            'description': description
        }
        
        response = requests.post(url, headers=headers, files=files, data=data)
        response.raise_for_status()
        return response.json()
    
    def delete_audience(self, audience_id: str) -> Dict[str, Any]:
        """
        Delete an audience by ID.
        
        Args:
            audience_id: The audience ID to delete
            
        Returns:
            Dict with status
        """
        import requests
        url = f"{self._get_base_url()}/audience/{audience_id}"
        headers = self._get_auth_headers()
        
        response = requests.delete(url, headers=headers)
        response.raise_for_status()
        return response.json()
    
    def get_audience_members(
        self,
        audience_id: str,
        page: int = 1,
        offset: int = 10
    ) -> Dict[str, Any]:
        """
        Get members of an audience.
        
        Args:
            audience_id: The audience ID
            page: Page number (default: 1)
            offset: Items per page (default: 10)
            
        Returns:
            Dict with status and members data
        """
        import requests
        url = f"{self._get_base_url()}/audience/{audience_id}/members"
        headers = self._get_auth_headers()
        params = {"page": str(page), "offset": str(offset)}
        
        response = requests.get(url, headers=headers, params=params)
        response.raise_for_status()
        return response.json()
    
    def add_audience_members(
        self,
        audience_id: str,
        members: List[Dict[str, str]]
    ) -> Dict[str, Any]:
        """
        Add members to an existing audience.
        
        Args:
            audience_id: The audience ID
            members: List of member dicts with phoneNumber and optional firstName, lastName
            
        Returns:
            Dict with status and count of added/skipped members
            
        Example:
            result = client.add_audience_members(
                audience_id="123abc",
                members=[
                    {"phoneNumber": "+919353662554", "firstName": "New", "lastName": "User"}
                ]
            )
        """
        import requests
        url = f"{self._get_base_url()}/audience/{audience_id}/members"
        headers = self._get_auth_headers()
        
        response = requests.post(url, headers=headers, json={"members": members})
        response.raise_for_status()
        return response.json()
    
    # ================================================================
    # Phone Number Methods
    # ================================================================
    
    def get_phone_numbers(self) -> Dict[str, Any]:
        """
        Get all acquired phone numbers for the organization.
        
        Returns:
            Dict with status and list of phone number products.
            Each product has:
            - _id: Phone number ID (use this in campaigns)
            - attributes.phoneNumber: The actual phone number
            - attributes.provider: Telephony provider
            - agentId: Assigned agent ID (if any)
            
        Example:
            numbers = client.get_phone_numbers()
            for num in numbers["data"]:
                print(f"ID: {num['_id']}, Phone: {num['attributes']['phoneNumber']}")
        """
        import requests
        url = f"{self._get_base_url()}/product/phone-numbers"
        headers = self._get_auth_headers()
        
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.json()

    # ================================================================
    # Simplified Wrapper Methods (cleaner API)
    # ================================================================
    
    def new_agent(
        self,
        name: str,
        prompt: str = None,
        description: str = "",
        kb_id: str = None,
        background_sound: str = ""
    ) -> Dict[str, Any]:
        """
        Create a new agent with simple parameters.
        
        Args:
            name: Agent name
            prompt: Global prompt for the agent
            description: Agent description
            kb_id: Knowledge base ID to link
            background_sound: '', 'office', 'cafe', 'call_center', or 'static'
        
        Returns:
            Dict with status and agent ID
        """
        from smallestai.atoms.models import CreateAgentRequest
        
        request = CreateAgentRequest(
            name=name,
            description=description,
            global_prompt=prompt,
            global_knowledge_base_id=kb_id,
            background_sound=background_sound
        )
        return self.create_agent(create_agent_request=request)
    
    def new_campaign(
        self,
        name: str,
        agent_id: str,
        audience_id: str,
        phone_ids: list,
        description: str = "",
        max_retries: int = 3,
        retry_delay: int = 15
    ) -> Dict[str, Any]:
        """
        Create a new campaign with simple parameters.
        
        Args:
            name: Campaign name
            agent_id: ID of the agent to use
            audience_id: ID of the audience to call
            phone_ids: List of outbound phone number IDs
            description: Campaign description
            max_retries: Max retry attempts (0-10)
            retry_delay: Minutes between retries (1-1440)
        
        Returns:
            Dict with campaign details
        """
        from smallestai.atoms.models import CampaignPostRequest
        
        request = CampaignPostRequest(
            name=name,
            description=description,
            agent_id=agent_id,
            audience_id=audience_id,
            phone_number_ids=phone_ids,
            max_retries=max_retries,
            retry_delay=retry_delay
        )
        return self.create_campaign(campaign_post_request=request)
    
    def new_kb(
        self,
        name: str,
        description: str = ""
    ) -> Dict[str, Any]:
        """
        Create a new knowledge base with simple parameters.
        
        Args:
            name: Knowledge base name
            description: Knowledge base description
        
        Returns:
            Dict with status and KB ID
        """
        from smallestai.atoms.models import KnowledgebasePostRequest
        
        request = KnowledgebasePostRequest(
            name=name,
            description=description
        )
        return self.create_knowledge_base(knowledgebase_post_request=request)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.serialization import FieldMetadata
from ....core.unchecked_base_model import UncheckedBaseModel


class DeleteVoiceCloneResponse(UncheckedBaseModel):
    success: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Status if the voice clone was deleted successfully.
    """

    voice_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="voiceId"),
        pydantic.Field(alias="voiceId", description="Voice ID of the deleted voice clone."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel
from .pronunciation_item import PronunciationItem


class PronunciationDict(UncheckedBaseModel):
    id: str = pydantic.Field()
    """
    Unique identifier for the pronunciation dictionary
    """

    items: typing.List[PronunciationItem] = pydantic.Field()
    """
    List of word-pronunciation pairs
    """

    created_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAt"),
        pydantic.Field(alias="createdAt", description="Timestamp when the dictionary was created"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""CLI chat application for communicating with Atoms SDK agents via WebSocket."""

import typer
from rich.console import Console

from smallestai.cli.agent import initialise_agent_app
from smallestai.cli.auth import initialise_auth_app
from smallestai.cli.lib.atoms import AtomsAPIClient
from smallestai.cli.lib.auth import AuthClient
from smallestai.cli.lib.project_config import ProjectConfig

console = Console()

app = typer.Typer(help="SmallestAI CLI")

auth_client = AuthClient()
atoms_client = AtomsAPIClient()
project_config = ProjectConfig()

agent_app = initialise_agent_app(project_config, auth_client, atoms_client)
app.add_typer(agent_app)

auth_app = initialise_auth_app(auth_client, atoms_client)
app.add_typer(auth_app)


def main():
    app()


if __name__ == "__main__":
    main()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel


class SinglePromptConfigToolsItemResponseVariablesItem(UncheckedBaseModel):
    variable_name: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="variableName"),
        pydantic.Field(alias="variableName", description="Name of the variable to store the extracted value"),
    ]
    json_path: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="jsonPath"),
        pydantic.Field(alias="jsonPath", description="JSON path to extract the value from response"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import contextlib
import typing
from json.decoder import JSONDecodeError
from logging import error, warning

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.http_sse._api import EventSource
from ...core.pydantic_utilities import parse_sse_obj
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.lightning_v31request_language import LightningV31RequestLanguage
from ..types.lightning_v31request_output_format import LightningV31RequestOutputFormat

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTextToSpeechClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextlib.contextmanager
    def synthesize_lightning_v31speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[int] = OMIT,
        speed: typing.Optional[float] = OMIT,
        language: typing.Optional[LightningV31RequestLanguage] = OMIT,
        output_format: typing.Optional[LightningV31RequestOutputFormat] = OMIT,
        pronunciation_dicts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[bytes]]]:
        """
        Converts provided text to speech using the Lightning V3.1 model.

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[int]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        language : typing.Optional[LightningV31RequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read in English. If set to 'hi', numbers will be read in Hindi.

        output_format : typing.Optional[LightningV31RequestOutputFormat]
            The format of the output audio.

        pronunciation_dicts : typing.Optional[typing.Sequence[str]]
            The IDs of the pronunciation dictionaries to use for speech generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[HttpResponse[typing.Iterator[bytes]]]
            Synthesized speech retrieved successfully.
        """
        with self._client_wrapper.httpx_client.stream(
            "api/v1/lightning-v3.1/get_speech",
            base_url=self._client_wrapper.get_environment().waves,
            method="POST",
            json={
                "text": text,
                "voice_id": voice_id,
                "sample_rate": sample_rate,
                "speed": speed,
                "language": language,
                "output_format": output_format,
                "pronunciation_dicts": pronunciation_dicts,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", None) if request_options is not None else None
                        return HttpResponse(
                            response=_response, data=(_chunk for _chunk in _response.iter_bytes(chunk_size=_chunk_size))
                        )
                    _response.read()
                    if _response.status_code == 400:
                        raise BadRequestError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 401:
                        raise UnauthorizedError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 500:
                        raise InternalServerError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()

    @contextlib.contextmanager
    def stream_lightning_v31speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[int] = OMIT,
        speed: typing.Optional[float] = OMIT,
        language: typing.Optional[LightningV31RequestLanguage] = OMIT,
        output_format: typing.Optional[LightningV31RequestOutputFormat] = OMIT,
        pronunciation_dicts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[typing.Any]]]:
        """
        Converts provided text to speech using the Lightning V3.1 model with streaming response.

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[int]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        language : typing.Optional[LightningV31RequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read in English. If set to 'hi', numbers will be read in Hindi.

        output_format : typing.Optional[LightningV31RequestOutputFormat]
            The format of the output audio.

        pronunciation_dicts : typing.Optional[typing.Sequence[str]]
            The IDs of the pronunciation dictionaries to use for speech generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[HttpResponse[typing.Iterator[typing.Any]]]
            Synthesized speech retrieved successfully.
        """
        with self._client_wrapper.httpx_client.stream(
            "api/v1/lightning-v3.1/stream",
            base_url=self._client_wrapper.get_environment().waves,
            method="POST",
            json={
                "text": text,
                "voice_id": voice_id,
                "sample_rate": sample_rate,
                "speed": speed,
                "language": language,
                "output_format": output_format,
                "pronunciation_dicts": pronunciation_dicts,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[typing.Any]]:
                try:
                    if 200 <= _response.status_code < 300:

                        def _iter():
                            _event_source = EventSource(_response)
                            for _sse in _event_source.iter_sse():
                                if _sse.data == None:
                                    return
                                try:
                                    yield typing.cast(
                                        typing.Any,
                                        parse_sse_obj(
                                            sse=_sse,
                                            type_=typing.Any,  # type: ignore
                                        ),
                                    )
                                except JSONDecodeError as e:
                                    warning(f"Skipping SSE event with invalid JSON: {e}, sse: {_sse!r}")
                                except (TypeError, ValueError, KeyError, AttributeError) as e:
                                    warning(
                                        f"Skipping SSE event due to model construction error: {type(e).__name__}: {e}, sse: {_sse!r}"
                                    )
                                except Exception as e:
                                    error(
                                        f"Unexpected error processing SSE event: {type(e).__name__}: {e}, sse: {_sse!r}"
                                    )
                            return

                        return HttpResponse(response=_response, data=_iter())
                    _response.read()
                    if _response.status_code == 400:
                        raise BadRequestError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 401:
                        raise UnauthorizedError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 500:
                        raise InternalServerError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()


class AsyncRawTextToSpeechClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextlib.asynccontextmanager
    async def synthesize_lightning_v31speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[int] = OMIT,
        speed: typing.Optional[float] = OMIT,
        language: typing.Optional[LightningV31RequestLanguage] = OMIT,
        output_format: typing.Optional[LightningV31RequestOutputFormat] = OMIT,
        pronunciation_dicts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]:
        """
        Converts provided text to speech using the Lightning V3.1 model.

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[int]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        language : typing.Optional[LightningV31RequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read in English. If set to 'hi', numbers will be read in Hindi.

        output_format : typing.Optional[LightningV31RequestOutputFormat]
            The format of the output audio.

        pronunciation_dicts : typing.Optional[typing.Sequence[str]]
            The IDs of the pronunciation dictionaries to use for speech generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[bytes]]]
            Synthesized speech retrieved successfully.
        """
        async with self._client_wrapper.httpx_client.stream(
            "api/v1/lightning-v3.1/get_speech",
            base_url=self._client_wrapper.get_environment().waves,
            method="POST",
            json={
                "text": text,
                "voice_id": voice_id,
                "sample_rate": sample_rate,
                "speed": speed,
                "language": language,
                "output_format": output_format,
                "pronunciation_dicts": pronunciation_dicts,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[bytes]]:
                try:
                    if 200 <= _response.status_code < 300:
                        _chunk_size = request_options.get("chunk_size", None) if request_options is not None else None
                        return AsyncHttpResponse(
                            response=_response,
                            data=(_chunk async for _chunk in _response.aiter_bytes(chunk_size=_chunk_size)),
                        )
                    await _response.aread()
                    if _response.status_code == 400:
                        raise BadRequestError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 401:
                        raise UnauthorizedError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 500:
                        raise InternalServerError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()

    @contextlib.asynccontextmanager
    async def stream_lightning_v31speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[int] = OMIT,
        speed: typing.Optional[float] = OMIT,
        language: typing.Optional[LightningV31RequestLanguage] = OMIT,
        output_format: typing.Optional[LightningV31RequestOutputFormat] = OMIT,
        pronunciation_dicts: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[typing.Any]]]:
        """
        Converts provided text to speech using the Lightning V3.1 model with streaming response.

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[int]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        language : typing.Optional[LightningV31RequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read in English. If set to 'hi', numbers will be read in Hindi.

        output_format : typing.Optional[LightningV31RequestOutputFormat]
            The format of the output audio.

        pronunciation_dicts : typing.Optional[typing.Sequence[str]]
            The IDs of the pronunciation dictionaries to use for speech generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[typing.Any]]]
            Synthesized speech retrieved successfully.
        """
        async with self._client_wrapper.httpx_client.stream(
            "api/v1/lightning-v3.1/stream",
            base_url=self._client_wrapper.get_environment().waves,
            method="POST",
            json={
                "text": text,
                "voice_id": voice_id,
                "sample_rate": sample_rate,
                "speed": speed,
                "language": language,
                "output_format": output_format,
                "pronunciation_dicts": pronunciation_dicts,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[typing.Any]]:
                try:
                    if 200 <= _response.status_code < 300:

                        async def _iter():
                            _event_source = EventSource(_response)
                            async for _sse in _event_source.aiter_sse():
                                if _sse.data == None:
                                    return
                                try:
                                    yield typing.cast(
                                        typing.Any,
                                        parse_sse_obj(
                                            sse=_sse,
                                            type_=typing.Any,  # type: ignore
                                        ),
                                    )
                                except JSONDecodeError as e:
                                    warning(f"Skipping SSE event with invalid JSON: {e}, sse: {_sse!r}")
                                except (TypeError, ValueError, KeyError, AttributeError) as e:
                                    warning(
                                        f"Skipping SSE event due to model construction error: {type(e).__name__}: {e}, sse: {_sse!r}"
                                    )
                                except Exception as e:
                                    error(
                                        f"Unexpected error processing SSE event: {type(e).__name__}: {e}, sse: {_sse!r}"
                                    )
                            return

                        return AsyncHttpResponse(response=_response, data=_iter())
                    await _response.aread()
                    if _response.status_code == 400:
                        raise BadRequestError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 401:
                        raise UnauthorizedError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 500:
                        raise InternalServerError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                construct_type(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()



# coding: utf-8

# flake8: noqa
"""
    Agent Management API

    API for managing agents, their templates, and call logs

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from smallestai.atoms.models.agent_agent_id_webhook_subscriptions_delete200_response import AgentAgentIdWebhookSubscriptionsDelete200Response
from smallestai.atoms.models.agent_agent_id_webhook_subscriptions_get200_response import AgentAgentIdWebhookSubscriptionsGet200Response
from smallestai.atoms.models.agent_agent_id_webhook_subscriptions_get404_response import AgentAgentIdWebhookSubscriptionsGet404Response
from smallestai.atoms.models.agent_agent_id_webhook_subscriptions_post201_response import AgentAgentIdWebhookSubscriptionsPost201Response
from smallestai.atoms.models.agent_agent_id_webhook_subscriptions_post400_response import AgentAgentIdWebhookSubscriptionsPost400Response
from smallestai.atoms.models.agent_agent_id_webhook_subscriptions_post_request import AgentAgentIdWebhookSubscriptionsPostRequest
from smallestai.atoms.models.agent_dto import AgentDTO
from smallestai.atoms.models.agent_dto_language import AgentDTOLanguage
from smallestai.atoms.models.agent_dto_language_switching import AgentDTOLanguageSwitching
from smallestai.atoms.models.agent_dto_synthesizer import AgentDTOSynthesizer
from smallestai.atoms.models.agent_dto_synthesizer_voice_config import AgentDTOSynthesizerVoiceConfig
from smallestai.atoms.models.agent_from_template_post200_response import AgentFromTemplatePost200Response
from smallestai.atoms.models.agent_get200_response import AgentGet200Response
from smallestai.atoms.models.agent_get200_response_data import AgentGet200ResponseData
from smallestai.atoms.models.agent_id_delete200_response import AgentIdDelete200Response
from smallestai.atoms.models.agent_id_get200_response import AgentIdGet200Response
from smallestai.atoms.models.agent_id_patch200_response import AgentIdPatch200Response
from smallestai.atoms.models.agent_id_patch_request import AgentIdPatchRequest
from smallestai.atoms.models.agent_id_patch_request_language import AgentIdPatchRequestLanguage
from smallestai.atoms.models.agent_id_patch_request_language_switching import AgentIdPatchRequestLanguageSwitching
from smallestai.atoms.models.agent_id_patch_request_synthesizer import AgentIdPatchRequestSynthesizer
from smallestai.atoms.models.agent_id_patch_request_synthesizer_voice_config import AgentIdPatchRequestSynthesizerVoiceConfig
from smallestai.atoms.models.agent_id_patch_request_synthesizer_voice_config_one_of import AgentIdPatchRequestSynthesizerVoiceConfigOneOf
from smallestai.atoms.models.agent_id_patch_request_synthesizer_voice_config_one_of1 import AgentIdPatchRequestSynthesizerVoiceConfigOneOf1
from smallestai.atoms.models.agent_id_workflow_get200_response import AgentIdWorkflowGet200Response
from smallestai.atoms.models.agent_id_workflow_get200_response_data import AgentIdWorkflowGet200ResponseData
from smallestai.atoms.models.agent_id_workflow_get200_response_data_edges_inner import AgentIdWorkflowGet200ResponseDataEdgesInner
from smallestai.atoms.models.agent_id_workflow_get200_response_data_edges_inner_data import AgentIdWorkflowGet200ResponseDataEdgesInnerData
from smallestai.atoms.models.agent_id_workflow_get200_response_data_edges_inner_marker_end import AgentIdWorkflowGet200ResponseDataEdgesInnerMarkerEnd
from smallestai.atoms.models.agent_id_workflow_get200_response_data_nodes_inner import AgentIdWorkflowGet200ResponseDataNodesInner
from smallestai.atoms.models.agent_id_workflow_get200_response_data_nodes_inner_data import AgentIdWorkflowGet200ResponseDataNodesInnerData
from smallestai.atoms.models.agent_id_workflow_get200_response_data_nodes_inner_data_variables import AgentIdWorkflowGet200ResponseDataNodesInnerDataVariables
from smallestai.atoms.models.agent_id_workflow_get200_response_data_nodes_inner_data_variables_data_inner import AgentIdWorkflowGet200ResponseDataNodesInnerDataVariablesDataInner
from smallestai.atoms.models.agent_id_workflow_get200_response_data_nodes_inner_position import AgentIdWorkflowGet200ResponseDataNodesInnerPosition
from smallestai.atoms.models.agent_id_workflow_get404_response import AgentIdWorkflowGet404Response
from smallestai.atoms.models.agent_template_get200_response import AgentTemplateGet200Response
from smallestai.atoms.models.agent_template_get200_response_data_inner import AgentTemplateGet200ResponseDataInner
from smallestai.atoms.models.api_response import ApiResponse
from smallestai.atoms.models.audience_get200_response import AudienceGet200Response
from smallestai.atoms.models.audience_get200_response_data_inner import AudienceGet200ResponseDataInner
from smallestai.atoms.models.audience_id_delete200_response import AudienceIdDelete200Response
from smallestai.atoms.models.audience_id_delete400_response import AudienceIdDelete400Response
from smallestai.atoms.models.audience_id_get200_response import AudienceIdGet200Response
from smallestai.atoms.models.audience_id_get400_response import AudienceIdGet400Response
from smallestai.atoms.models.audience_id_get403_response import AudienceIdGet403Response
from smallestai.atoms.models.audience_id_get404_response import AudienceIdGet404Response
from smallestai.atoms.models.audience_id_members_delete200_response import AudienceIdMembersDelete200Response
from smallestai.atoms.models.audience_id_members_delete200_response_data import AudienceIdMembersDelete200ResponseData
from smallestai.atoms.models.audience_id_members_delete_request import AudienceIdMembersDeleteRequest
from smallestai.atoms.models.audience_id_members_get200_response import AudienceIdMembersGet200Response
from smallestai.atoms.models.audience_id_members_get200_response_data import AudienceIdMembersGet200ResponseData
from smallestai.atoms.models.audience_id_members_get200_response_data_members_inner import AudienceIdMembersGet200ResponseDataMembersInner
from smallestai.atoms.models.audience_id_members_get400_response import AudienceIdMembersGet400Response
from smallestai.atoms.models.audience_id_members_get500_response import AudienceIdMembersGet500Response
from smallestai.atoms.models.audience_id_members_post200_response import AudienceIdMembersPost200Response
from smallestai.atoms.models.audience_id_members_post200_response_data_inner import AudienceIdMembersPost200ResponseDataInner
from smallestai.atoms.models.audience_id_members_post200_response_data_inner_data import AudienceIdMembersPost200ResponseDataInnerData
from smallestai.atoms.models.audience_id_members_post400_response import AudienceIdMembersPost400Response
from smallestai.atoms.models.audience_id_members_post403_response import AudienceIdMembersPost403Response
from smallestai.atoms.models.audience_id_members_post_request import AudienceIdMembersPostRequest
from smallestai.atoms.models.audience_id_members_search_get200_response import AudienceIdMembersSearchGet200Response
from smallestai.atoms.models.audience_id_members_search_get200_response_data import AudienceIdMembersSearchGet200ResponseData
from smallestai.atoms.models.audience_id_members_search_get200_response_data_search_info import AudienceIdMembersSearchGet200ResponseDataSearchInfo
from smallestai.atoms.models.audience_id_members_search_get400_response import AudienceIdMembersSearchGet400Response
from smallestai.atoms.models.audience_id_members_search_get500_response import AudienceIdMembersSearchGet500Response
from smallestai.atoms.models.audience_post200_response import AudiencePost200Response
from smallestai.atoms.models.audience_post200_response_data import AudiencePost200ResponseData
from smallestai.atoms.models.audience_post400_response import AudiencePost400Response
from smallestai.atoms.models.bad_request_error_response import BadRequestErrorResponse
from smallestai.atoms.models.campaign_get200_response import CampaignGet200Response
from smallestai.atoms.models.campaign_get200_response_data import CampaignGet200ResponseData
from smallestai.atoms.models.campaign_get_request import CampaignGetRequest
from smallestai.atoms.models.campaign_id_get200_response import CampaignIdGet200Response
from smallestai.atoms.models.campaign_id_get200_response_data import CampaignIdGet200ResponseData
from smallestai.atoms.models.campaign_post201_response import CampaignPost201Response
from smallestai.atoms.models.campaign_post_request import CampaignPostRequest
from smallestai.atoms.models.conversation_id_get200_response import ConversationIdGet200Response
from smallestai.atoms.models.conversation_id_get200_response_data import ConversationIdGet200ResponseData
from smallestai.atoms.models.conversation_outbound_post200_response import ConversationOutboundPost200Response
from smallestai.atoms.models.conversation_outbound_post200_response_data import ConversationOutboundPost200ResponseData
from smallestai.atoms.models.conversation_outbound_post_request import ConversationOutboundPostRequest
from smallestai.atoms.models.create_agent_from_template_request import CreateAgentFromTemplateRequest
from smallestai.atoms.models.create_agent_request import CreateAgentRequest
from smallestai.atoms.models.create_agent_request_language import CreateAgentRequestLanguage
from smallestai.atoms.models.create_agent_request_language_synthesizer import CreateAgentRequestLanguageSynthesizer
from smallestai.atoms.models.create_agent_request_language_synthesizer_voice_config import CreateAgentRequestLanguageSynthesizerVoiceConfig
from smallestai.atoms.models.internal_server_error_response import InternalServerErrorResponse
from smallestai.atoms.models.knowledge_base import KnowledgeBase
from smallestai.atoms.models.knowledge_base_item import KnowledgeBaseItem
from smallestai.atoms.models.knowledgebase_get200_response import KnowledgebaseGet200Response
from smallestai.atoms.models.knowledgebase_id_get200_response import KnowledgebaseIdGet200Response
from smallestai.atoms.models.knowledgebase_id_items_get200_response import KnowledgebaseIdItemsGet200Response
from smallestai.atoms.models.knowledgebase_id_items_upload_text_post_request import KnowledgebaseIdItemsUploadTextPostRequest
from smallestai.atoms.models.knowledgebase_post201_response import KnowledgebasePost201Response
from smallestai.atoms.models.knowledgebase_post_request import KnowledgebasePostRequest
from smallestai.atoms.models.organization_get200_response import OrganizationGet200Response
from smallestai.atoms.models.organization_get200_response_data import OrganizationGet200ResponseData
from smallestai.atoms.models.organization_get200_response_data_members_inner import OrganizationGet200ResponseDataMembersInner
from smallestai.atoms.models.organization_get200_response_data_subscription import OrganizationGet200ResponseDataSubscription
from smallestai.atoms.models.product_phone_numbers_get200_response import ProductPhoneNumbersGet200Response
from smallestai.atoms.models.product_phone_numbers_get200_response_data_inner import ProductPhoneNumbersGet200ResponseDataInner
from smallestai.atoms.models.product_phone_numbers_get200_response_data_inner_attributes import ProductPhoneNumbersGet200ResponseDataInnerAttributes
from smallestai.atoms.models.unauthorized_error_reponse import UnauthorizedErrorReponse
from smallestai.atoms.models.user_get200_response import UserGet200Response
from smallestai.atoms.models.user_get200_response_data import UserGet200ResponseData
from smallestai.atoms.models.webhook import Webhook
from smallestai.atoms.models.webhook_agent import WebhookAgent
from smallestai.atoms.models.webhook_event import WebhookEvent
from smallestai.atoms.models.webhook_get200_response import WebhookGet200Response
from smallestai.atoms.models.webhook_get200_response_data import WebhookGet200ResponseData
from smallestai.atoms.models.webhook_id_delete404_response import WebhookIdDelete404Response
from smallestai.atoms.models.webhook_post201_response import WebhookPost201Response
from smallestai.atoms.models.webhook_post_request import WebhookPostRequest
from smallestai.atoms.models.webhook_post_request_events_inner import WebhookPostRequestEventsInner
from smallestai.atoms.models.webhook_subscription import WebhookSubscription
from smallestai.atoms.models.webhook_subscription_populated import WebhookSubscriptionPopulated



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.unchecked_base_model import UncheckedBaseModel


class PulseTranscriptionResponseMessageWordsItem(UncheckedBaseModel):
    word: typing.Optional[str] = pydantic.Field(default=None)
    """
    The transcribed word
    """

    start: typing.Optional[float] = pydantic.Field(default=None)
    """
    Start time in seconds
    """

    end: typing.Optional[float] = pydantic.Field(default=None)
    """
    End time in seconds
    """

    confidence: typing.Optional[float] = pydantic.Field(default=None)
    """
    Confidence score for the word (0.0 to 1.0)
    """

    speaker: typing.Optional[int] = pydantic.Field(default=None)
    """
    Speaker label (when diarization is enabled)
    """

    speaker_confidence: typing.Optional[float] = pydantic.Field(default=None)
    """
    Confidence score for the speaker assignment (0.0 to 1.0)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from . import (
        asr_streaming,
        lightning,
        lightning_asr_streaming,
        lightning_large,
        lightning_v2,
        lightning_v2tts,
        lightning_v31tts,
        pronunciation_dictionaries,
        pulse_stt_streaming,
        speech_to_text,
        streaming_tts,
        text_to_speech,
        voice_cloning,
        voices,
    )
    from .async_waves_client import AsyncWavesClient
    from .waves_client import WavesClient
    from .stream_tts import WavesStreamingTTS, TTSConfig
_dynamic_imports: typing.Dict[str, str] = {
    "asr_streaming": ".asr_streaming",
    "lightning": ".lightning",
    "lightning_asr_streaming": ".lightning_asr_streaming",
    "lightning_large": ".lightning_large",
    "lightning_v2": ".lightning_v2",
    "lightning_v2tts": ".lightning_v2tts",
    "lightning_v31tts": ".lightning_v31tts",
    "pronunciation_dictionaries": ".pronunciation_dictionaries",
    "pulse_stt_streaming": ".pulse_stt_streaming",
    "speech_to_text": ".speech_to_text",
    "streaming_tts": ".streaming_tts",
    "text_to_speech": ".text_to_speech",
    "voice_cloning": ".voice_cloning",
    "voices": ".voices",
    # High-level helper re-exported for convenience:
    #   from smallestai.waves import AsyncWavesClient
    "AsyncWavesClient": ".async_waves_client",
    # Production clients from v4.3.1:
    "WavesClient": ".waves_client",
    "WavesStreamingTTS": ".stream_tts",
    "TTSConfig": ".stream_tts",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "asr_streaming",
    "lightning",
    "lightning_asr_streaming",
    "lightning_large",
    "lightning_v2",
    "lightning_v2tts",
    "lightning_v31tts",
    "pronunciation_dictionaries",
    "pulse_stt_streaming",
    "speech_to_text",
    "streaming_tts",
    "text_to_speech",
    "voice_cloning",
    "voices",
    "AsyncWavesClient",
    "WavesClient",
    "WavesStreamingTTS",
    "TTSConfig",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawCampaignsClient, RawCampaignsClient
from .types.delete_campaign_id_response import DeleteCampaignIdResponse
from .types.get_campaign_id_response import GetCampaignIdResponse
from .types.get_campaign_response import GetCampaignResponse
from .types.post_campaign_id_pause_response import PostCampaignIdPauseResponse
from .types.post_campaign_id_start_response import PostCampaignIdStartResponse
from .types.post_campaign_response import PostCampaignResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CampaignsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCampaignsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCampaignsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCampaignsClient
        """
        return self._raw_client

    def retrieve_all_campaigns(self, *, request_options: typing.Optional[RequestOptions] = None) -> GetCampaignResponse:
        """
        Get all campaigns

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCampaignResponse
            A list of campaigns

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.campaigns.retrieve_all_campaigns()
        """
        _response = self._raw_client.retrieve_all_campaigns(request_options=request_options)
        return _response.data

    def create_a_campaign(
        self,
        *,
        name: str,
        audience_id: str,
        agent_id: str,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostCampaignResponse:
        """
        Create a campaign

        Parameters
        ----------
        name : str
            The name of the campaign

        audience_id : str
            The ID of the audience

        agent_id : str
            The ID of the agent

        description : typing.Optional[str]
            The description of the campaign

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostCampaignResponse
            Campaign created successfully

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.campaigns.create_a_campaign(
            name="My Campaign",
            audience_id="60d0fe4f5311236168a109ca",
            agent_id="60d0fe4f5311236168a109ca",
        )
        """
        _response = self._raw_client.create_a_campaign(
            name=name,
            audience_id=audience_id,
            agent_id=agent_id,
            description=description,
            request_options=request_options,
        )
        return _response.data

    def get_a_campaign(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCampaignIdResponse:
        """
        Get a campaign with detailed metrics

        Parameters
        ----------
        id : str
            The ID of the campaign

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCampaignIdResponse
            Campaign details with metrics

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.campaigns.get_a_campaign(
            id="id",
        )
        """
        _response = self._raw_client.get_a_campaign(id, request_options=request_options)
        return _response.data

    def delete_a_campaign(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteCampaignIdResponse:
        """
        Delete a campaign

        Parameters
        ----------
        id : str
            The ID of the campaign

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCampaignIdResponse
            Campaign deleted successfully

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.campaigns.delete_a_campaign(
            id="id",
        )
        """
        _response = self._raw_client.delete_a_campaign(id, request_options=request_options)
        return _response.data

    def start_a_campaign(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PostCampaignIdStartResponse:
        """
        Start a campaign

        Parameters
        ----------
        id : str
            The ID of the campaign

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostCampaignIdStartResponse
            Campaign started successfully

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.campaigns.start_a_campaign(
            id="id",
        )
        """
        _response = self._raw_client.start_a_campaign(id, request_options=request_options)
        return _response.data

    def pause_a_campaign(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PostCampaignIdPauseResponse:
        """
        Pause a campaign

        Parameters
        ----------
        id : str
            The ID of the campaign

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostCampaignIdPauseResponse
            Campaign paused successfully

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.campaigns.pause_a_campaign(
            id="id",
        )
        """
        _response = self._raw_client.pause_a_campaign(id, request_options=request_options)
        return _response.data


class AsyncCampaignsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCampaignsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCampaignsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCampaignsClient
        """
        return self._raw_client

    async def retrieve_all_campaigns(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCampaignResponse:
        """
        Get all campaigns

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCampaignResponse
            A list of campaigns

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.campaigns.retrieve_all_campaigns()


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve_all_campaigns(request_options=request_options)
        return _response.data

    async def create_a_campaign(
        self,
        *,
        name: str,
        audience_id: str,
        agent_id: str,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostCampaignResponse:
        """
        Create a campaign

        Parameters
        ----------
        name : str
            The name of the campaign

        audience_id : str
            The ID of the audience

        agent_id : str
            The ID of the agent

        description : typing.Optional[str]
            The description of the campaign

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostCampaignResponse
            Campaign created successfully

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.campaigns.create_a_campaign(
                name="My Campaign",
                audience_id="60d0fe4f5311236168a109ca",
                agent_id="60d0fe4f5311236168a109ca",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_a_campaign(
            name=name,
            audience_id=audience_id,
            agent_id=agent_id,
            description=description,
            request_options=request_options,
        )
        return _response.data

    async def get_a_campaign(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCampaignIdResponse:
        """
        Get a campaign with detailed metrics

        Parameters
        ----------
        id : str
            The ID of the campaign

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCampaignIdResponse
            Campaign details with metrics

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.campaigns.get_a_campaign(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_a_campaign(id, request_options=request_options)
        return _response.data

    async def delete_a_campaign(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteCampaignIdResponse:
        """
        Delete a campaign

        Parameters
        ----------
        id : str
            The ID of the campaign

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCampaignIdResponse
            Campaign deleted successfully

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.campaigns.delete_a_campaign(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_a_campaign(id, request_options=request_options)
        return _response.data

    async def start_a_campaign(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PostCampaignIdStartResponse:
        """
        Start a campaign

        Parameters
        ----------
        id : str
            The ID of the campaign

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostCampaignIdStartResponse
            Campaign started successfully

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.campaigns.start_a_campaign(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.start_a_campaign(id, request_options=request_options)
        return _response.data

    async def pause_a_campaign(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PostCampaignIdPauseResponse:
        """
        Pause a campaign

        Parameters
        ----------
        id : str
            The ID of the campaign

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostCampaignIdPauseResponse
            Campaign paused successfully

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.campaigns.pause_a_campaign(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.pause_a_campaign(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.unauthorized_error import UnauthorizedError
from .types.get_conversation_id_response import GetConversationIdResponse
from .types.get_conversation_request_call_types import GetConversationRequestCallTypes
from .types.get_conversation_response import GetConversationResponse
from .types.post_conversation_search_response import PostConversationSearchResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawLogsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_all_conversation_logs(
        self,
        *,
        page: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        agent_ids: typing.Optional[str] = None,
        campaign_ids: typing.Optional[str] = None,
        call_types: typing.Optional[GetConversationRequestCallTypes] = None,
        search: typing.Optional[str] = None,
        status_filter: typing.Optional[str] = None,
        disconnect_reason_filter: typing.Optional[str] = None,
        call_attempt_filter: typing.Optional[str] = None,
        duration_filter: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetConversationResponse]:
        """
        Retrieve paginated conversation logs with support for various filters. Returns call logs for agents belonging to the authenticated user's organization.

        Parameters
        ----------
        page : typing.Optional[int]
            Page number for pagination

        limit : typing.Optional[int]
            Number of items per page

        agent_ids : typing.Optional[str]
            Comma-separated list of agent IDs to filter by

        campaign_ids : typing.Optional[str]
            Comma-separated list of campaign IDs to filter by

        call_types : typing.Optional[GetConversationRequestCallTypes]
            Comma-separated list of call types to filter by

        search : typing.Optional[str]
            Search query to filter by callId, fromNumber, or toNumber

        status_filter : typing.Optional[str]
            Comma-separated list of call statuses to filter by.
            Available statuses: pending, in_progress, completed, failed, no_answer, cancelled, busy

        disconnect_reason_filter : typing.Optional[str]
            Comma-separated list of disconnect reasons to filter by.
            Available reasons: user_hangup, agent_hangup, connection_error, timeout, system_error, transfer_complete

        call_attempt_filter : typing.Optional[str]
            Comma-separated list of call attempt types to filter by.
            Available filters: initial (first attempt calls), retry (retry attempt calls), all (all calls)

        duration_filter : typing.Optional[str]
            Comma-separated list of duration ranges to filter by.
            Available ranges: 0-30 (0-30 seconds), 30-60 (30-60 seconds), 1-5 (1-5 minutes), 5+ (more than 5 minutes)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetConversationResponse]
            Successful response
        """
        _response = self._client_wrapper.httpx_client.request(
            "conversation",
            base_url=self._client_wrapper.get_environment().atoms,
            method="GET",
            params={
                "page": page,
                "limit": limit,
                "agentIds": agent_ids,
                "campaignIds": campaign_ids,
                "callTypes": call_types,
                "search": search,
                "statusFilter": status_filter,
                "disconnectReasonFilter": disconnect_reason_filter,
                "callAttemptFilter": call_attempt_filter,
                "durationFilter": duration_filter,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConversationResponse,
                    construct_type(
                        type_=GetConversationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def search_conversation_logs_by_call_i_ds(
        self, *, call_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PostConversationSearchResponse]:
        """
        Fetch specific conversation logs by their callIds. This endpoint allows you to retrieve up to 100 specific calls at once.
        Only returns calls that belong to agents in your organization (security check enforced).
        Unlike the GET /conversation endpoint, this endpoint can also return retry calls (non-root calls).

        Parameters
        ----------
        call_ids : typing.Sequence[str]
            Array of callIds to fetch

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PostConversationSearchResponse]
            Successful response
        """
        _response = self._client_wrapper.httpx_client.request(
            "conversation/search",
            base_url=self._client_wrapper.get_environment().atoms,
            method="POST",
            json={
                "callIds": call_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostConversationSearchResponse,
                    construct_type(
                        type_=PostConversationSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_conversation_log_by_id(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetConversationIdResponse]:
        """
        Retrieve detailed information about a specific conversation including transcript, events, and latency metrics.

        Parameters
        ----------
        id : str
            The callId of the conversation. You can get the callId from the conversation logs endpoint.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetConversationIdResponse]
            Successful response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"conversation/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().atoms,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConversationIdResponse,
                    construct_type(
                        type_=GetConversationIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLogsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_all_conversation_logs(
        self,
        *,
        page: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        agent_ids: typing.Optional[str] = None,
        campaign_ids: typing.Optional[str] = None,
        call_types: typing.Optional[GetConversationRequestCallTypes] = None,
        search: typing.Optional[str] = None,
        status_filter: typing.Optional[str] = None,
        disconnect_reason_filter: typing.Optional[str] = None,
        call_attempt_filter: typing.Optional[str] = None,
        duration_filter: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetConversationResponse]:
        """
        Retrieve paginated conversation logs with support for various filters. Returns call logs for agents belonging to the authenticated user's organization.

        Parameters
        ----------
        page : typing.Optional[int]
            Page number for pagination

        limit : typing.Optional[int]
            Number of items per page

        agent_ids : typing.Optional[str]
            Comma-separated list of agent IDs to filter by

        campaign_ids : typing.Optional[str]
            Comma-separated list of campaign IDs to filter by

        call_types : typing.Optional[GetConversationRequestCallTypes]
            Comma-separated list of call types to filter by

        search : typing.Optional[str]
            Search query to filter by callId, fromNumber, or toNumber

        status_filter : typing.Optional[str]
            Comma-separated list of call statuses to filter by.
            Available statuses: pending, in_progress, completed, failed, no_answer, cancelled, busy

        disconnect_reason_filter : typing.Optional[str]
            Comma-separated list of disconnect reasons to filter by.
            Available reasons: user_hangup, agent_hangup, connection_error, timeout, system_error, transfer_complete

        call_attempt_filter : typing.Optional[str]
            Comma-separated list of call attempt types to filter by.
            Available filters: initial (first attempt calls), retry (retry attempt calls), all (all calls)

        duration_filter : typing.Optional[str]
            Comma-separated list of duration ranges to filter by.
            Available ranges: 0-30 (0-30 seconds), 30-60 (30-60 seconds), 1-5 (1-5 minutes), 5+ (more than 5 minutes)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetConversationResponse]
            Successful response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "conversation",
            base_url=self._client_wrapper.get_environment().atoms,
            method="GET",
            params={
                "page": page,
                "limit": limit,
                "agentIds": agent_ids,
                "campaignIds": campaign_ids,
                "callTypes": call_types,
                "search": search,
                "statusFilter": status_filter,
                "disconnectReasonFilter": disconnect_reason_filter,
                "callAttemptFilter": call_attempt_filter,
                "durationFilter": duration_filter,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConversationResponse,
                    construct_type(
                        type_=GetConversationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def search_conversation_logs_by_call_i_ds(
        self, *, call_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PostConversationSearchResponse]:
        """
        Fetch specific conversation logs by their callIds. This endpoint allows you to retrieve up to 100 specific calls at once.
        Only returns calls that belong to agents in your organization (security check enforced).
        Unlike the GET /conversation endpoint, this endpoint can also return retry calls (non-root calls).

        Parameters
        ----------
        call_ids : typing.Sequence[str]
            Array of callIds to fetch

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PostConversationSearchResponse]
            Successful response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "conversation/search",
            base_url=self._client_wrapper.get_environment().atoms,
            method="POST",
            json={
                "callIds": call_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostConversationSearchResponse,
                    construct_type(
                        type_=PostConversationSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_conversation_log_by_id(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetConversationIdResponse]:
        """
        Retrieve detailed information about a specific conversation including transcript, events, and latency metrics.

        Parameters
        ----------
        id : str
            The callId of the conversation. You can get the callId from the conversation logs endpoint.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetConversationIdResponse]
            Successful response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"conversation/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().atoms,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConversationIdResponse,
                    construct_type(
                        type_=GetConversationIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawLightningClient, RawLightningClient
from .types.lightning_request_language import LightningRequestLanguage
from .types.lightning_request_output_format import LightningRequestOutputFormat
from ..types.sample_rate import SampleRate

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class LightningClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawLightningClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawLightningClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawLightningClient
        """
        return self._raw_client

    def synthesize_lightning_speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[SampleRate] = OMIT,
        speed: typing.Optional[float] = OMIT,
        language: typing.Optional[LightningRequestLanguage] = OMIT,
        output_format: typing.Optional[LightningRequestOutputFormat] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Get speech for given text using the Waves API

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[SampleRate]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        language : typing.Optional[LightningRequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read as individual digits in English. If set to 'hi', numbers will be read as individual digits in Hindi.

        output_format : typing.Optional[LightningRequestOutputFormat]
            The format of the output audio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            Synthesized speech retrieved successfully.

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.waves.lightning.synthesize_lightning_speech(
            text="text",
            voice_id="voice_id",
        )
        """
        with self._raw_client.synthesize_lightning_speech(
            text=text,
            voice_id=voice_id,
            sample_rate=sample_rate,
            speed=speed,
            language=language,
            output_format=output_format,
            request_options=request_options,
        ) as r:
            yield from r.data


class AsyncLightningClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawLightningClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawLightningClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawLightningClient
        """
        return self._raw_client

    async def synthesize_lightning_speech(
        self,
        *,
        text: str,
        voice_id: str,
        sample_rate: typing.Optional[SampleRate] = OMIT,
        speed: typing.Optional[float] = OMIT,
        language: typing.Optional[LightningRequestLanguage] = OMIT,
        output_format: typing.Optional[LightningRequestOutputFormat] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Get speech for given text using the Waves API

        Parameters
        ----------
        text : str
            The text to convert to speech.

        voice_id : str
            The voice identifier to use for speech generation.

        sample_rate : typing.Optional[SampleRate]
            The sample rate for the generated audio.

        speed : typing.Optional[float]
            The speed of the generated speech.

        language : typing.Optional[LightningRequestLanguage]
            Determines how numbers are spelled out. If set to 'en', numbers will be read as individual digits in English. If set to 'hi', numbers will be read as individual digits in Hindi.

        output_format : typing.Optional[LightningRequestOutputFormat]
            The format of the output audio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            Synthesized speech retrieved successfully.

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.waves.lightning.synthesize_lightning_speech(
                text="text",
                voice_id="voice_id",
            )


        asyncio.run(main())
        """
        async with self._raw_client.synthesize_lightning_speech(
            text=text,
            voice_id=voice_id,
            sample_rate=sample_rate,
            speed=speed,
            language=language,
            output_format=output_format,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk



# This file was auto-generated by Fern from our API Definition.

import typing

GetCampaignIdResponseDataCampaignStatus = typing.Union[
    typing.Literal["draft", "scheduled", "running", "paused", "scheduled_for_retry", "completed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.serialization import FieldMetadata
from ....core.unchecked_base_model import UncheckedBaseModel


class GetConversationIdResponseDataVoiceConfigUsed(UncheckedBaseModel):
    """
    The voice configuration that was actually used for this call.
    """

    model: typing.Optional[str] = pydantic.Field(default=None)
    """
    The TTS model used for the call.
    """

    voice_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="voiceId"),
        pydantic.Field(alias="voiceId", description="The voice ID used for the call."),
    ] = None
    gender: typing.Optional[str] = pydantic.Field(default=None)
    """
    The gender of the voice used for the call (e.g., "male", "female").
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

GetConversationResponseDataLogsItemStatus = typing.Union[
    typing.Literal["pending", "in_progress", "completed", "failed", "no_answer", "cancelled", "busy"], typing.Any
]



"""
Specialized base node classes for different agent behaviors.
"""

from smallestai.atoms.agent.nodes.background_agent import BackgroundAgentNode
from smallestai.atoms.agent.nodes.base import Node
from smallestai.atoms.agent.nodes.output_agent import OutputAgentNode

__all__ = ["BackgroundAgentNode", "OutputAgentNode", "Node"]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.delete_pronunciation_dict_response import DeletePronunciationDictResponse
from ..types.pronunciation_dict import PronunciationDict
from ..types.pronunciation_item import PronunciationItem
from ..types.update_pronunciation_dict_response import UpdatePronunciationDictResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPronunciationDictionariesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_pronunciation_dicts(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[PronunciationDict]]:
        """
        Retrieve all pronunciation dictionaries for the authenticated user

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[PronunciationDict]]
            List of pronunciation dictionaries
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/pronunciation-dicts",
            base_url=self._client_wrapper.get_environment().waves,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[PronunciationDict],
                    construct_type(
                        type_=typing.List[PronunciationDict],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_pronunciation_dict(
        self, *, items: typing.Sequence[PronunciationItem], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PronunciationDict]:
        """
        Create a new pronunciation dictionary for the authenticated user

        Parameters
        ----------
        items : typing.Sequence[PronunciationItem]
            List of word-pronunciation pairs to create

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PronunciationDict]
            Successfully created pronunciation dictionary
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/pronunciation-dicts",
            base_url=self._client_wrapper.get_environment().waves,
            method="POST",
            json={
                "items": convert_and_respect_annotation_metadata(
                    object_=items, annotation=typing.Sequence[PronunciationItem], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PronunciationDict,
                    construct_type(
                        type_=PronunciationDict,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_pronunciation_dict(
        self,
        *,
        id: str,
        items: typing.Sequence[PronunciationItem],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdatePronunciationDictResponse]:
        """
        Update an existing pronunciation dictionary for the authenticated user

        Parameters
        ----------
        id : str
            ID of the pronunciation dictionary to update

        items : typing.Sequence[PronunciationItem]
            Updated list of word-pronunciation pairs

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdatePronunciationDictResponse]
            Successfully updated pronunciation dictionary
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/pronunciation-dicts",
            base_url=self._client_wrapper.get_environment().waves,
            method="PUT",
            json={
                "id": id,
                "items": convert_and_respect_annotation_metadata(
                    object_=items, annotation=typing.Sequence[PronunciationItem], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdatePronunciationDictResponse,
                    construct_type(
                        type_=UpdatePronunciationDictResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_pronunciation_dict(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeletePronunciationDictResponse]:
        """
        Delete an existing pronunciation dictionary for the authenticated user

        Parameters
        ----------
        id : str
            ID of the pronunciation dictionary to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeletePronunciationDictResponse]
            Successfully deleted pronunciation dictionary
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/pronunciation-dicts",
            base_url=self._client_wrapper.get_environment().waves,
            method="DELETE",
            json={
                "id": id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeletePronunciationDictResponse,
                    construct_type(
                        type_=DeletePronunciationDictResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawPronunciationDictionariesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_pronunciation_dicts(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[PronunciationDict]]:
        """
        Retrieve all pronunciation dictionaries for the authenticated user

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[PronunciationDict]]
            List of pronunciation dictionaries
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/pronunciation-dicts",
            base_url=self._client_wrapper.get_environment().waves,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[PronunciationDict],
                    construct_type(
                        type_=typing.List[PronunciationDict],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_pronunciation_dict(
        self, *, items: typing.Sequence[PronunciationItem], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PronunciationDict]:
        """
        Create a new pronunciation dictionary for the authenticated user

        Parameters
        ----------
        items : typing.Sequence[PronunciationItem]
            List of word-pronunciation pairs to create

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PronunciationDict]
            Successfully created pronunciation dictionary
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/pronunciation-dicts",
            base_url=self._client_wrapper.get_environment().waves,
            method="POST",
            json={
                "items": convert_and_respect_annotation_metadata(
                    object_=items, annotation=typing.Sequence[PronunciationItem], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PronunciationDict,
                    construct_type(
                        type_=PronunciationDict,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_pronunciation_dict(
        self,
        *,
        id: str,
        items: typing.Sequence[PronunciationItem],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdatePronunciationDictResponse]:
        """
        Update an existing pronunciation dictionary for the authenticated user

        Parameters
        ----------
        id : str
            ID of the pronunciation dictionary to update

        items : typing.Sequence[PronunciationItem]
            Updated list of word-pronunciation pairs

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdatePronunciationDictResponse]
            Successfully updated pronunciation dictionary
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/pronunciation-dicts",
            base_url=self._client_wrapper.get_environment().waves,
            method="PUT",
            json={
                "id": id,
                "items": convert_and_respect_annotation_metadata(
                    object_=items, annotation=typing.Sequence[PronunciationItem], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdatePronunciationDictResponse,
                    construct_type(
                        type_=UpdatePronunciationDictResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_pronunciation_dict(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeletePronunciationDictResponse]:
        """
        Delete an existing pronunciation dictionary for the authenticated user

        Parameters
        ----------
        id : str
            ID of the pronunciation dictionary to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeletePronunciationDictResponse]
            Successfully deleted pronunciation dictionary
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/pronunciation-dicts",
            base_url=self._client_wrapper.get_environment().waves,
            method="DELETE",
            json={
                "id": id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeletePronunciationDictResponse,
                    construct_type(
                        type_=DeletePronunciationDictResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel
from .webhook_status import WebhookStatus
from .webhook_subscription_populated import WebhookSubscriptionPopulated


class Webhook(UncheckedBaseModel):
    id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="_id"),
        pydantic.Field(alias="_id", description="The unique identifier for the webhook"),
    ] = None
    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The webhook endpoint URL
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the webhook
    """

    status: typing.Optional[WebhookStatus] = pydantic.Field(default=None)
    """
    The status of the webhook
    """

    organization_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="organizationId"),
        pydantic.Field(alias="organizationId", description="The organization ID"),
    ] = None
    created_by: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="createdBy"),
        pydantic.Field(alias="createdBy", description="The user ID who created the webhook"),
    ] = None
    subscriptions: typing.Optional[typing.List[WebhookSubscriptionPopulated]] = pydantic.Field(default=None)
    """
    A list of subscriptions for the webhook with populated agent details.
    """

    decrypted_secret_key: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="decryptedSecretKey"),
        pydantic.Field(
            alias="decryptedSecretKey",
            description="The decrypted signing secret for the webhook. This is only returned when fetching a single webhook by ID.",
        ),
    ] = None
    created_at: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="createdAt"),
        pydantic.Field(alias="createdAt", description="The date and time when the webhook was created"),
    ] = None
    updated_at: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="updatedAt"),
        pydantic.Field(alias="updatedAt", description="The date and time when the webhook was last updated"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawAtomsClient, RawAtomsClient

if typing.TYPE_CHECKING:
    from .agent_templates.client import AgentTemplatesClient, AsyncAgentTemplatesClient
    from .agents.client import AgentsClient, AsyncAgentsClient
    from .audience.client import AsyncAudienceClient, AudienceClient
    from .calls.client import AsyncCallsClient, CallsClient
    from .campaigns.client import AsyncCampaignsClient, CampaignsClient
    from .knowledge_base.client import AsyncKnowledgeBaseClient, KnowledgeBaseClient
    from .logs.client import AsyncLogsClient, LogsClient
    from .organization.client import AsyncOrganizationClient, OrganizationClient
    from .phone_numbers.client import AsyncPhoneNumbersClient, PhoneNumbersClient
    from .user.client import AsyncUserClient, UserClient
    from .webhooks.client import AsyncWebhooksClient, WebhooksClient
    from .workflows.client import AsyncWorkflowsClient, WorkflowsClient


class AtomsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAtomsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._user: typing.Optional[UserClient] = None
        self._organization: typing.Optional[OrganizationClient] = None
        self._agent_templates: typing.Optional[AgentTemplatesClient] = None
        self._agents: typing.Optional[AgentsClient] = None
        self._workflows: typing.Optional[WorkflowsClient] = None
        self._logs: typing.Optional[LogsClient] = None
        self._calls: typing.Optional[CallsClient] = None
        self._campaigns: typing.Optional[CampaignsClient] = None
        self._knowledge_base: typing.Optional[KnowledgeBaseClient] = None
        self._phone_numbers: typing.Optional[PhoneNumbersClient] = None
        self._webhooks: typing.Optional[WebhooksClient] = None
        self._audience: typing.Optional[AudienceClient] = None

    @property
    def with_raw_response(self) -> RawAtomsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAtomsClient
        """
        return self._raw_client

    @property
    def user(self):
        if self._user is None:
            from .user.client import UserClient  # noqa: E402

            self._user = UserClient(client_wrapper=self._client_wrapper)
        return self._user

    @property
    def organization(self):
        if self._organization is None:
            from .organization.client import OrganizationClient  # noqa: E402

            self._organization = OrganizationClient(client_wrapper=self._client_wrapper)
        return self._organization

    @property
    def agent_templates(self):
        if self._agent_templates is None:
            from .agent_templates.client import AgentTemplatesClient  # noqa: E402

            self._agent_templates = AgentTemplatesClient(client_wrapper=self._client_wrapper)
        return self._agent_templates

    @property
    def agents(self):
        if self._agents is None:
            from .agents.client import AgentsClient  # noqa: E402

            self._agents = AgentsClient(client_wrapper=self._client_wrapper)
        return self._agents

    @property
    def workflows(self):
        if self._workflows is None:
            from .workflows.client import WorkflowsClient  # noqa: E402

            self._workflows = WorkflowsClient(client_wrapper=self._client_wrapper)
        return self._workflows

    @property
    def logs(self):
        if self._logs is None:
            from .logs.client import LogsClient  # noqa: E402

            self._logs = LogsClient(client_wrapper=self._client_wrapper)
        return self._logs

    @property
    def calls(self):
        if self._calls is None:
            from .calls.client import CallsClient  # noqa: E402

            self._calls = CallsClient(client_wrapper=self._client_wrapper)
        return self._calls

    @property
    def campaigns(self):
        if self._campaigns is None:
            from .campaigns.client import CampaignsClient  # noqa: E402

            self._campaigns = CampaignsClient(client_wrapper=self._client_wrapper)
        return self._campaigns

    @property
    def knowledge_base(self):
        if self._knowledge_base is None:
            from .knowledge_base.client import KnowledgeBaseClient  # noqa: E402

            self._knowledge_base = KnowledgeBaseClient(client_wrapper=self._client_wrapper)
        return self._knowledge_base

    @property
    def phone_numbers(self):
        if self._phone_numbers is None:
            from .phone_numbers.client import PhoneNumbersClient  # noqa: E402

            self._phone_numbers = PhoneNumbersClient(client_wrapper=self._client_wrapper)
        return self._phone_numbers

    @property
    def webhooks(self):
        if self._webhooks is None:
            from .webhooks.client import WebhooksClient  # noqa: E402

            self._webhooks = WebhooksClient(client_wrapper=self._client_wrapper)
        return self._webhooks

    @property
    def audience(self):
        if self._audience is None:
            from .audience.client import AudienceClient  # noqa: E402

            self._audience = AudienceClient(client_wrapper=self._client_wrapper)
        return self._audience


class AsyncAtomsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAtomsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._user: typing.Optional[AsyncUserClient] = None
        self._organization: typing.Optional[AsyncOrganizationClient] = None
        self._agent_templates: typing.Optional[AsyncAgentTemplatesClient] = None
        self._agents: typing.Optional[AsyncAgentsClient] = None
        self._workflows: typing.Optional[AsyncWorkflowsClient] = None
        self._logs: typing.Optional[AsyncLogsClient] = None
        self._calls: typing.Optional[AsyncCallsClient] = None
        self._campaigns: typing.Optional[AsyncCampaignsClient] = None
        self._knowledge_base: typing.Optional[AsyncKnowledgeBaseClient] = None
        self._phone_numbers: typing.Optional[AsyncPhoneNumbersClient] = None
        self._webhooks: typing.Optional[AsyncWebhooksClient] = None
        self._audience: typing.Optional[AsyncAudienceClient] = None

    @property
    def with_raw_response(self) -> AsyncRawAtomsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAtomsClient
        """
        return self._raw_client

    @property
    def user(self):
        if self._user is None:
            from .user.client import AsyncUserClient  # noqa: E402

            self._user = AsyncUserClient(client_wrapper=self._client_wrapper)
        return self._user

    @property
    def organization(self):
        if self._organization is None:
            from .organization.client import AsyncOrganizationClient  # noqa: E402

            self._organization = AsyncOrganizationClient(client_wrapper=self._client_wrapper)
        return self._organization

    @property
    def agent_templates(self):
        if self._agent_templates is None:
            from .agent_templates.client import AsyncAgentTemplatesClient  # noqa: E402

            self._agent_templates = AsyncAgentTemplatesClient(client_wrapper=self._client_wrapper)
        return self._agent_templates

    @property
    def agents(self):
        if self._agents is None:
            from .agents.client import AsyncAgentsClient  # noqa: E402

            self._agents = AsyncAgentsClient(client_wrapper=self._client_wrapper)
        return self._agents

    @property
    def workflows(self):
        if self._workflows is None:
            from .workflows.client import AsyncWorkflowsClient  # noqa: E402

            self._workflows = AsyncWorkflowsClient(client_wrapper=self._client_wrapper)
        return self._workflows

    @property
    def logs(self):
        if self._logs is None:
            from .logs.client import AsyncLogsClient  # noqa: E402

            self._logs = AsyncLogsClient(client_wrapper=self._client_wrapper)
        return self._logs

    @property
    def calls(self):
        if self._calls is None:
            from .calls.client import AsyncCallsClient  # noqa: E402

            self._calls = AsyncCallsClient(client_wrapper=self._client_wrapper)
        return self._calls

    @property
    def campaigns(self):
        if self._campaigns is None:
            from .campaigns.client import AsyncCampaignsClient  # noqa: E402

            self._campaigns = AsyncCampaignsClient(client_wrapper=self._client_wrapper)
        return self._campaigns

    @property
    def knowledge_base(self):
        if self._knowledge_base is None:
            from .knowledge_base.client import AsyncKnowledgeBaseClient  # noqa: E402

            self._knowledge_base = AsyncKnowledgeBaseClient(client_wrapper=self._client_wrapper)
        return self._knowledge_base

    @property
    def phone_numbers(self):
        if self._phone_numbers is None:
            from .phone_numbers.client import AsyncPhoneNumbersClient  # noqa: E402

            self._phone_numbers = AsyncPhoneNumbersClient(client_wrapper=self._client_wrapper)
        return self._phone_numbers

    @property
    def webhooks(self):
        if self._webhooks is None:
            from .webhooks.client import AsyncWebhooksClient  # noqa: E402

            self._webhooks = AsyncWebhooksClient(client_wrapper=self._client_wrapper)
        return self._webhooks

    @property
    def audience(self):
        if self._audience is None:
            from .audience.client import AsyncAudienceClient  # noqa: E402

            self._audience = AsyncAudienceClient(client_wrapper=self._client_wrapper)
        return self._audience
